Imports System.Runtime.InteropServices

Namespace DirectShow
    Public Class VideoPlayer
        Private Graph As IGraphBuilder

        Private Audio As IBasicAudio
        Private Window As IVideoWindow
        Private Pos As IMediaPosition
        Private Control As IMediaControl
        Private Events As IMediaEventEx
        Private Seek As IMediaSeeking
        Private Video As IBasicVideo2

        Private State As PlayState

        Private Const ConstWS_CAPTION As Integer = &HC00000
        Private Const WM_GRAPHNOTIFY As Integer = &H8001
        Private Const WS_CHILD As Integer = &H40000000
        Private Const SW_SHOW As Integer = 5

        Public Sub OpenFile(ByVal path As String)
            CloseInterfaces()
            GetInterfaces(path)
        End Sub

        Public Sub Close()
            CloseInterfaces()
        End Sub

        Public Sub PlayPause()
            If State = State.Running Then
                Control.Pause()
                State = State.Pause
            Else
                Try
                    Control.Run()
                    State = PlayState.Running
                Catch ex As Exception
                End Try
            End If
        End Sub

        Public Sub SetWindowPosition(ByVal handle As IntPtr, ByVal left As Integer, ByVal top As Integer, ByVal width As Integer, ByVal height As Integer)
            Window.put_Owner(handle)
            Window.put_WindowStyle(WS_CHILD)
            Window.put_WindowState(SW_SHOW)
            Window.put_MessageDrain(handle)
            Window.SetWindowPosition(left, top, width, height)
        End Sub

        Public Function GetLengthInSeconds() As Integer
            Dim d As Double
            Pos.get_Duration(d)
            Return CInt(d)
        End Function

        Public Function GetFramerate() As Double
            Dim d As Double
            Pos.get_Duration(d)
            Return Math.Round(GetFrames() / d, 2)
        End Function

        Public Function GetFrames() As Integer
            Dim a As Double
            Video.AvgTimePerFrame(a)
            Dim d As Double
            Pos.get_Duration(d)
            If a = 0 Then Return 0
            Return CInt(d / a)
        End Function

        Public Function GetPositionInSeconds() As Integer
            Dim Position As Double
            Pos.get_CurrentPosition(Position)
            Return CInt(Position)
        End Function

        Public Function GetPositionInFrames() As Integer
            Try
                Dim Position As Double
                Pos.get_CurrentPosition(Position)
                Return CInt(Position * GetFramerate())
            Catch
            End Try
        End Function

        Public Sub SetPositionInFrames(ByVal frames As Integer)
            Pos.put_CurrentPosition(frames / GetFramerate())
            If State = PlayState.Pause Then Control.Pause()
        End Sub

        Public Sub SetPositionInSeconds(ByVal seconds As Integer)
            Pos.put_CurrentPosition(seconds)
            If State = PlayState.Pause Then Control.Pause()
        End Sub

        Public Sub StepFrames(ByVal frames As Integer)
            frames = GetPositionInFrames() + frames
            If frames < 0 Then
                SetPositionInFrames(0)
            ElseIf frames < GetFrames() Then
                SetPositionInFrames(frames)
            Else
                SetPositionInFrames(GetFrames() - 1)
            End If
        End Sub

        Public Function GetWidth() As Integer
            Dim Width As Integer
            Video.VideoWidth(Width)
            Return Width
        End Function

        Public Function GetHeight() As Integer
            Dim Height As Integer
            Video.VideoHeight(Height)
            Return Height
        End Function

        Private Function GetPin(ByRef filter As IBaseFilter, ByVal d As PinDirection) As IPin
            Dim EnumPins As IEnumPins
            Dim Direction As PinDirection
            Dim Fetched As Integer

            filter.EnumPins(EnumPins)
            EnumPins.Next(1, GetPin, Fetched)
            GetPin.QueryDirection(Direction)
            If Not Direction = d Then EnumPins.Next(1, GetPin, Fetched)
            If GetPin Is Nothing Then Msg.Show("Couldn't get pin", "Error", MessageBoxIcon.Error)
        End Function

        Private Sub GetInterfaces(ByVal path As String)
            Dim comtype As Type = Nothing
            Dim comobj As Object = Nothing

            comtype = Type.GetTypeFromCLSID(New Guid(&HE436EBB3, &H524F, &H11CE, &H9F, &H53, &H0, &H20, &HAF, &HB, &HA7, &H70))
            If comtype Is Nothing Then Throw New NotSupportedException("DirectX (8.1 or higher) not installed?")
            comobj = Activator.CreateInstance(comtype)
            Graph = CType(comobj, IGraphBuilder)
            comobj = Nothing

            Audio = CType(Graph, IBasicAudio)
            Window = CType(Graph, IVideoWindow)
            Pos = CType(Graph, IMediaPosition)
            Control = CType(Graph, IMediaControl)
            Events = CType(Graph, IMediaEventEx)
            Seek = CType(Graph, IMediaSeeking)
            Video = CType(Graph, IBasicVideo2)

            Graph.RenderFile(path, Nothing)

            If Not comobj Is Nothing Then
                Marshal.ReleaseComObject(comobj)
                comobj = Nothing
            End If
        End Sub

        Private Sub ShowHRError(ByVal hr As Integer)
            If hr > 0 Then System.Runtime.InteropServices.Marshal.ThrowExceptionForHR(hr)
        End Sub

        Private Sub CloseInterfaces()
            Window = Nothing
            Events = Nothing
            Control = Nothing
            Audio = Nothing
            Pos = Nothing
            Seek = Nothing
            Video = Nothing
            If Not Graph Is Nothing Then
                Marshal.ReleaseComObject(Graph)
                Graph = Nothing
            End If
            State = PlayState.Pause
        End Sub

        Private Enum PlayState
            Pause
            Running
        End Enum
    End Class

    <ComVisible(True), ComImport(), Guid("56a868a9-0ad4-11ce-b03a-0020af0ba770"), _
    InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
    Public Interface IGraphBuilder
        <PreserveSig()> Function AddFilter(<[In]()> ByVal pFilter As IBaseFilter, <[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal pName As String) As Integer
        <PreserveSig()> Function RemoveFilter(<[In]()> ByVal pFilter As IBaseFilter) As Integer
        <PreserveSig()> Function EnumFilters(<Out()> ByRef ppEnum As IEnumFilters) As Integer
        <PreserveSig()> Function FindFilterByName(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal pName As String, <Out()> ByRef ppFilter As IBaseFilter) As Integer
        <PreserveSig()> Function ConnectDirect(<[In]()> ByVal ppinOut As IPin, <[In]()> ByVal ppinIn As IPin, <[In](), MarshalAs(UnmanagedType.LPStruct)> ByVal pmt As AMMediaType) As Integer
        <PreserveSig()> Function Reconnect(<[In]()> ByVal ppin As IPin) As Integer
        <PreserveSig()> Function Disconnect(<[In]()> ByVal ppin As IPin) As Integer
        <PreserveSig()> Function SetDefaultSyncSource() As Integer
        <PreserveSig()> Function Connect(<[In]()> ByVal ppinOut As IPin, <[In]()> ByVal ppinIn As IPin) As Integer
        <PreserveSig()> Function Render(<[In]()> ByVal ppinOut As IPin) As Integer
        <PreserveSig()> Function RenderFile(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal lpcwstrFile As String, <[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal lpcwstrPlayList As String) As Integer
        <PreserveSig()> Function AddSourceFilter(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal lpcwstrFileName As String, <[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal lpcwstrFilterName As String, <Out()> ByRef ppFilter As IBaseFilter) As Integer
        <PreserveSig()> Function SetLogFile(ByVal hFile As IntPtr) As Integer
        <PreserveSig()> Function Abort() As Integer
        <PreserveSig()> Function houldOperationContinue() As Integer
    End Interface

    <ComVisible(True), ComImport(), Guid("56a86895-0ad4-11ce-b03a-0020af0ba770"), _
    InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
    Public Interface IBaseFilter
        <PreserveSig()> Function GetClassID(<Out()> ByRef pClassID As Guid) As Integer
        <PreserveSig()> Function [Stop]() As Integer
        <PreserveSig()> Function Pause() As Integer
        <PreserveSig()> Function Run(ByVal tStart As Long) As Integer
        <PreserveSig()> Function GetState(ByVal dwMilliSecsTimeout As Integer, <Out()> ByRef filtState As Integer) As Integer
        <PreserveSig()> Function SetSyncSource(<[In]()> ByVal pClock As IReferenceClock) As Integer
        <PreserveSig()> Function GetSyncSource(<Out()> ByRef pClock As IReferenceClock) As Integer
        <PreserveSig()> Function EnumPins(<Out()> ByRef ppEnum As IEnumPins) As Integer
        <PreserveSig()> Function FindPin(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal Id As String, <Out()> ByRef ppPin As IPin) As Integer
        <PreserveSig()> Function QueryFilterInfo(<Out()> ByVal pInfo As FilterInfo) As Integer
        <PreserveSig()> Function JoinFilterGraph(<[In]()> ByVal pGraph As IFilterGraph, <[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal pName As String) As Integer
        <PreserveSig()> Function QueryVendorInfo(<Out(), MarshalAs(UnmanagedType.LPWStr)> ByRef pVendorInfo As String) As Integer
    End Interface

    <ComVisible(True), ComImport(), Guid("56a86893-0ad4-11ce-b03a-0020af0ba770"), _
    InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
    Public Interface IEnumFilters
        <PreserveSig()> Function [Next](<[In]()> ByVal cFilters As Integer, <Out(), MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=2)> ByRef ppFilter As IBaseFilter(), <Out()> ByRef pcFetched As Integer) As Integer
        <PreserveSig()> Function Skip(<[In]()> ByVal cFilters As Integer) As Integer
        Sub Reset()
        Sub Clone(<Out()> ByRef ppEnum As IEnumFilters)
    End Interface

    <ComVisible(True), ComImport(), Guid("56a86891-0ad4-11ce-b03a-0020af0ba770"), _
    InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
    Public Interface IPin
        <PreserveSig()> Function Connect(ByVal pReceivePin As IPin, <MarshalAs(UnmanagedType.LPStruct)> ByVal pmt As AMMediaType) As Integer
        <PreserveSig()> Function ReceiveConnection(ByVal pReceivePin As IPin, <MarshalAs(UnmanagedType.LPStruct)> ByVal pmt As AMMediaType) As Integer
        <PreserveSig()> Function Disconnect() As Integer
        <PreserveSig()> Function ConnectedTo(ByRef ppPin As IPin) As Integer
        <PreserveSig()> Function ConnectionMediaType(<MarshalAs(UnmanagedType.LPStruct)> ByVal pmt As AMMediaType) As Integer
        <PreserveSig()> Function QueryPinInfo(ByVal pInfo As IntPtr) As Integer
        <PreserveSig()> Function QueryDirection(ByRef pPinDir As PinDirection) As Integer
        <PreserveSig()> Function QueryId(<Out(), MarshalAs(UnmanagedType.LPWStr)> ByRef Id As String) As Integer
        <PreserveSig()> Function QueryAccept(<[In](), MarshalAs(UnmanagedType.LPStruct)> ByVal pmt As AMMediaType) As Integer
        <PreserveSig()> Function EnumMediaTypes(ByVal ppEnum As IntPtr) As Integer
        <PreserveSig()> Function QueryInternalConnections(ByVal apPin As IntPtr, <[In](), Out()> ByRef nPin As Integer) As Integer
        <PreserveSig()> Function EndOfStream() As Integer
        <PreserveSig()> Function BeginFlush() As Integer
        <PreserveSig()> Function EndFlush() As Integer
        <PreserveSig()> Function NewSegment(ByVal tStart As Long, ByVal tStop As Long, ByVal dRate As Double) As Integer
    End Interface

    <StructLayout(LayoutKind.Sequential), ComVisible(False)> _
    Public Class AMMediaType
        Public majorType As Guid
        Public subType As Guid
        <MarshalAs(UnmanagedType.Bool)> _
        Public fixedSizeSamples As Boolean
        <MarshalAs(UnmanagedType.Bool)> _
        Public temporalCompression As Boolean
        Public sampleSize As Integer
        Public formatType As Guid
        Public unkPtr As IntPtr
        Public formatSize As Integer
        Public formatPtr As IntPtr
    End Class

    <ComVisible(True), ComImport(), Guid("56a86897-0ad4-11ce-b03a-0020af0ba770"), _
    InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
    Public Interface IReferenceClock
        <PreserveSig()> Function GetTime(ByRef pTime As Long) As Integer
        <PreserveSig()> Function AdviseTime(ByVal baseTime As Long, ByVal streamTime As Long, ByVal hEvent As IntPtr, ByRef pdwAdviseCookie As Integer) As Integer
        <PreserveSig()> Function AdvisePeriodic(ByVal startTime As Long, ByVal periodTime As Long, ByVal hSemaphore As IntPtr, ByRef pdwAdviseCookie As Integer) As Integer
        <PreserveSig()> Function Unadvise(ByVal dwAdviseCookie As Integer) As Integer
    End Interface

    <ComVisible(True), ComImport(), Guid("56a86892-0ad4-11ce-b03a-0020af0ba770"), _
    InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
    Public Interface IEnumPins
        <PreserveSig()> Function [Next](<[In]()> ByVal cPins As Integer, <Out()> ByRef ppPins As IPin, <Out()> ByRef pcFetched As Integer) As Integer
        <PreserveSig()> Function Skip(<[In]()> ByVal cPins As Integer) As Integer
        Sub Reset()
        Sub Clone(<Out()> ByRef ppEnum As IEnumPins)
    End Interface

    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Unicode), ComVisible(False)> _
    Public Class FilterInfo
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=128)> _
        Public achName As String
        <MarshalAs(UnmanagedType.IUnknown)> _
        Public pUnk As Object
    End Class

    <ComVisible(True), ComImport(), Guid("56a8689f-0ad4-11ce-b03a-0020af0ba770"), _
    InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
    Public Interface IFilterGraph
        <PreserveSig()> Function AddFilter(<[In]()> ByVal pFilter As IBaseFilter, <[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal pName As String) As Integer
        <PreserveSig()> Function RemoveFilter(<[In]()> ByVal pFilter As IBaseFilter) As Integer
        <PreserveSig()> Function EnumFilters(<Out()> ByRef ppEnum As IEnumFilters) As Integer
        <PreserveSig()> Function FindFilterByName(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal pName As String, <Out()> ByRef ppFilter As IBaseFilter) As Integer
        <PreserveSig()> Function ConnectDirect(<[In]()> ByVal ppinOut As IPin, <[In]()> ByVal ppinIn As IPin, <[In](), MarshalAs(UnmanagedType.LPStruct)> ByVal pmt As AMMediaType) As Integer
        <PreserveSig()> Function Reconnect(<[In]()> ByVal ppin As IPin) As Integer
        <PreserveSig()> Function Disconnect(<[In]()> ByVal ppin As IPin) As Integer
        <PreserveSig()> Function SetDefaultSyncSource() As Integer
    End Interface

    <ComVisible(True), ComImport(), Guid("56a868b3-0ad4-11ce-b03a-0020af0ba770"), _
    InterfaceType(ComInterfaceType.InterfaceIsDual)> _
    Public Interface IBasicAudio
        <PreserveSig()> Function put_Volume(ByVal lVolume As Integer) As Integer
        <PreserveSig()> Function get_Volume(ByRef plVolume As Integer) As Integer
        <PreserveSig()> Function put_Balance(ByVal lBalance As Integer) As Integer
        <PreserveSig()> Function get_Balance(ByRef plBalanceout As Integer) As Integer
    End Interface

    <ComVisible(True), ComImport(), Guid("56a868b4-0ad4-11ce-b03a-0020af0ba770"), _
    InterfaceType(ComInterfaceType.InterfaceIsDual)> _
    Public Interface IVideoWindow
        <PreserveSig()> Function put_Caption(ByVal caption As String) As Integer
        <PreserveSig()> Function get_Caption(<Out()> ByRef caption As String) As Integer
        <PreserveSig()> Function put_WindowStyle(ByVal windowStyle As Integer) As Integer
        <PreserveSig()> Function get_WindowStyle(ByRef windowStyle As Integer) As Integer
        <PreserveSig()> Function put_WindowStyleEx(ByVal windowStyleEx As Integer) As Integer
        <PreserveSig()> Function get_WindowStyleEx(ByRef windowStyleEx As Integer) As Integer
        <PreserveSig()> Function put_AutoShow(ByVal autoShow As Integer) As Integer
        <PreserveSig()> Function get_AutoShow(ByRef autoShow As Integer) As Integer
        <PreserveSig()> Function put_WindowState(ByVal windowState As Integer) As Integer
        <PreserveSig()> Function get_WindowState(ByRef windowState As Integer) As Integer
        <PreserveSig()> Function put_BackgroundPalette(ByVal backgroundPalette As Integer) As Integer
        <PreserveSig()> Function get_BackgroundPalette(ByRef backgroundPalette As Integer) As Integer
        <PreserveSig()> Function put_Visible(ByVal visible As Integer) As Integer
        <PreserveSig()> Function get_Visible(ByRef visible As Integer) As Integer
        <PreserveSig()> Function put_Left(ByVal left As Integer) As Integer
        <PreserveSig()> Function get_Left(ByRef left As Integer) As Integer
        <PreserveSig()> Function put_Width(ByVal width As Integer) As Integer
        <PreserveSig()> Function get_Width(ByRef width As Integer) As Integer
        <PreserveSig()> Function put_Top(ByVal top As Integer) As Integer
        <PreserveSig()> Function get_Top(ByRef top As Integer) As Integer
        <PreserveSig()> Function put_Height(ByVal height As Integer) As Integer
        <PreserveSig()> Function get_Height(ByRef height As Integer) As Integer
        <PreserveSig()> Function put_Owner(ByVal owner As IntPtr) As Integer
        <PreserveSig()> Function get_Owner(ByRef owner As IntPtr) As Integer
        <PreserveSig()> Function put_MessageDrain(ByVal drain As IntPtr) As Integer
        <PreserveSig()> Function get_MessageDrain(ByRef drain As IntPtr) As Integer
        <PreserveSig()> Function get_BorderColor(ByRef color As Integer) As Integer
        <PreserveSig()> Function put_BorderColor(ByVal color As Integer) As Integer
        <PreserveSig()> Function get_FullScreenMode(ByRef fullScreenMode As Integer) As Integer
        <PreserveSig()> Function put_FullScreenMode(ByVal fullScreenMode As Integer) As Integer
        <PreserveSig()> Function SetWindowForeground(ByVal focus As Integer) As Integer
        <PreserveSig()> Function NotifyOwnerMessage(ByVal hwnd As IntPtr, ByVal msg As Integer, ByVal wParam As IntPtr, ByVal lParam As IntPtr) As Integer
        <PreserveSig()> Function SetWindowPosition(ByVal left As Integer, ByVal top As Integer, ByVal width As Integer, ByVal height As Integer) As Integer
        <PreserveSig()> Function GetWindowPosition(ByRef left As Integer, ByRef top As Integer, ByRef width As Integer, ByRef height As Integer) As Integer
        <PreserveSig()> Function GetMinIdealImageSize(ByRef width As Integer, ByRef height As Integer) As Integer
        <PreserveSig()> Function GetMaxIdealImageSize(ByRef width As Integer, ByRef height As Integer) As Integer
        <PreserveSig()> Function GetRestorePosition(ByRef left As Integer, ByRef top As Integer, ByRef width As Integer, ByRef height As Integer) As Integer
        <PreserveSig()> Function HideCursor(ByVal hideCursor1 As Integer) As Integer
        <PreserveSig()> Function IsCursorHidden(ByRef hideCursor As Integer) As Integer
    End Interface

    <ComVisible(True), ComImport(), Guid("56a868b2-0ad4-11ce-b03a-0020af0ba770"), _
    InterfaceType(ComInterfaceType.InterfaceIsDual)> _
    Public Interface IMediaPosition
        <PreserveSig()> Function get_Duration(ByRef pLength As Double) As Integer
        <PreserveSig()> Function put_CurrentPosition(ByVal llTime As Double) As Integer
        <PreserveSig()> Function get_CurrentPosition(ByRef pllTime As Double) As Integer
        <PreserveSig()> Function get_StopTime(ByRef pllTime As Double) As Integer
        <PreserveSig()> Function put_StopTime(ByVal llTime As Double) As Integer
        <PreserveSig()> Function get_PrerollTime(ByRef pllTime As Double) As Integer
        <PreserveSig()> Function put_PrerollTime(ByVal llTime As Double) As Integer
        <PreserveSig()> Function put_Rate(ByVal dRate As Double) As Integer
        <PreserveSig()> Function get_Rate(ByRef pdRate As Double) As Integer
        <PreserveSig()> Function CanSeekForward(ByRef pCanSeekForward As Integer) As Integer
        <PreserveSig()> Function CanSeekBackward(ByRef pCanSeekBackward As Integer) As Integer
    End Interface

    <ComVisible(True), ComImport(), Guid("56a868b1-0ad4-11ce-b03a-0020af0ba770"), _
    InterfaceType(ComInterfaceType.InterfaceIsDual)> _
    Public Interface IMediaControl
        <PreserveSig()> Function Run() As Integer
        <PreserveSig()> Function Pause() As Integer
        <PreserveSig()> Function [Stop]() As Integer
        <PreserveSig()> Function GetState(ByVal msTimeout As Integer, ByRef pfs As Integer) As Integer
        <PreserveSig()> Function RenderFile(ByVal strFilename As String) As Integer
        <PreserveSig()> Function AddSourceFilter(<[In]()> ByVal strFilename As String, <Out(), MarshalAs(UnmanagedType.IDispatch)> ByRef ppUnk As Object) As Integer
        <PreserveSig()> Function get_FilterCollection(<Out(), MarshalAs(UnmanagedType.IDispatch)> ByRef ppUnk As Object) As Integer
        <PreserveSig()> Function get_RegFilterCollection(<Out(), MarshalAs(UnmanagedType.IDispatch)> ByRef ppUnk As Object) As Integer
        <PreserveSig()> Function StopWhenReady() As Integer
    End Interface


    <ComVisible(True), ComImport(), Guid("56a868c0-0ad4-11ce-b03a-0020af0ba770"), _
     InterfaceType(ComInterfaceType.InterfaceIsDual)> _
    Public Interface IMediaEventEx
        <PreserveSig()> Function GetEventHandle(ByRef hEvent As IntPtr) As Integer
        <PreserveSig()> Function GetEvent(ByRef lEventCode As DsEvCode, ByRef lParam1 As Integer, ByRef lParam2 As Integer, ByVal msTimeout As Integer) As Integer
        <PreserveSig()> Function WaitForCompletion(ByVal msTimeout As Integer, <Out()> ByRef pEvCode As Integer) As Integer
        <PreserveSig()> Function CancelDefaultHandling(ByVal lEvCode As Integer) As Integer
        <PreserveSig()> Function RestoreDefaultHandling(ByVal lEvCode As Integer) As Integer
        <PreserveSig()> Function FreeEventParams(ByVal lEvCode As DsEvCode, ByVal lParam1 As Integer, ByVal lParam2 As Integer) As Integer
        <PreserveSig()> Function SetNotifyWindow(ByVal hwnd As IntPtr, ByVal lMsg As Integer, ByVal lInstanceData As IntPtr) As Integer
        <PreserveSig()> Function SetNotifyFlags(ByVal lNoNotifyFlags As Integer) As Integer
        <PreserveSig()> Function GetNotifyFlags(ByRef lplNoNotifyFlags As Integer) As Integer
    End Interface

    <ComVisible(True), ComImport(), Guid("36b73880-c2c8-11cf-8b46-00805f6cef60"), _
     InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
    Public Interface IMediaSeeking
        <PreserveSig()> Function GetCapabilities(ByRef pCapabilities As SeekingCapabilities) As Integer
        <PreserveSig()> Function CheckCapabilities(<[In](), Out()> ByRef pCapabilities As SeekingCapabilities) As Integer
        <PreserveSig()> Function IsFormatSupported(<[In]()> ByRef pFormat As Guid) As Integer
        <PreserveSig()> Function QueryPreferredFormat(<Out()> ByRef pFormat As Guid) As Integer
        <PreserveSig()> Function GetTimeFormat(<Out()> ByRef pFormat As Guid) As Integer
        <PreserveSig()> Function IsUsingTimeFormat(<[In]()> ByRef pFormat As Guid) As Integer
        <PreserveSig()> Function SetTimeFormat(<[In]()> ByRef pFormat As Guid) As Integer
        <PreserveSig()> Function GetDuration(ByRef pDuration As Long) As Integer
        <PreserveSig()> Function GetStopPosition(ByRef pStop As Long) As Integer
        <PreserveSig()> Function GetCurrentPosition(ByRef pCurrent As Long) As Integer
        <PreserveSig()> Function ConvertTimeFormat(ByRef pTarget As Long, <[In]()> ByRef pTargetFormat As Guid, ByVal Source As Long, <[In]()> ByRef pSourceFormat As Guid) As Integer
        <PreserveSig()> Function SetPositions(<[In](), Out(), MarshalAs(UnmanagedType.LPStruct)> ByVal pCurrent As DsOptInt64, ByVal dwCurrentFlags As SeekingFlags, <[In](), Out(), MarshalAs(UnmanagedType.LPStruct)> ByVal pStop As DsOptInt64, ByVal dwStopFlags As SeekingFlags) As Integer
        <PreserveSig()> Function GetPositions(ByRef pCurrent As Long, ByRef pStop As Long) As Integer
        <PreserveSig()> Function GetAvailable(ByRef pEarliest As Long, ByRef pLatest As Long) As Integer
        <PreserveSig()> Function SetRate(ByVal dRate As Double) As Integer
        <PreserveSig()> Function GetRate(ByRef pdRate As Double) As Integer
        <PreserveSig()> Function GetPreroll(ByRef pllPreroll As Long) As Integer
    End Interface

    <ComVisible(True), ComImport(), Guid("329bb360-f6ea-11d1-9038-00a0c9697298"), _
    InterfaceType(ComInterfaceType.InterfaceIsDual)> _
    Public Interface IBasicVideo2
        <PreserveSig()> Function AvgTimePerFrame(ByRef pAvgTimePerFrame As Double) As Integer
        <PreserveSig()> Function BitRate(ByRef pBitRate As Integer) As Integer
        <PreserveSig()> Function BitErrorRate(ByRef pBitRate As Integer) As Integer
        <PreserveSig()> Function VideoWidth(ByRef pVideoWidth As Integer) As Integer
        <PreserveSig()> Function VideoHeight(ByRef pVideoHeight As Integer) As Integer
        <PreserveSig()> Function put_SourceLeft(ByVal SourceLeft As Integer) As Integer
        <PreserveSig()> Function get_SourceLeft(ByRef pSourceLeft As Integer) As Integer
        <PreserveSig()> Function put_SourceWidth(ByVal SourceWidth As Integer) As Integer
        <PreserveSig()> Function get_SourceWidth(ByRef pSourceWidth As Integer) As Integer
        <PreserveSig()> Function put_SourceTop(ByVal SourceTop As Integer) As Integer
        <PreserveSig()> Function get_SourceTop(ByRef pSourceTop As Integer) As Integer
        <PreserveSig()> Function put_SourceHeight(ByVal SourceHeight As Integer) As Integer
        <PreserveSig()> Function get_SourceHeight(ByRef pSourceHeight As Integer) As Integer
        <PreserveSig()> Function put_DestinationLeft(ByVal DestinationLeft As Integer) As Integer
        <PreserveSig()> Function get_DestinationLeft(ByRef pDestinationLeft As Integer) As Integer
        <PreserveSig()> Function put_DestinationWidth(ByVal DestinationWidth As Integer) As Integer
        <PreserveSig()> Function get_DestinationWidth(ByRef pDestinationWidth As Integer) As Integer
        <PreserveSig()> Function put_DestinationTop(ByVal DestinationTop As Integer) As Integer
        <PreserveSig()> Function get_DestinationTop(ByRef pDestinationTop As Integer) As Integer
        <PreserveSig()> Function put_DestinationHeight(ByVal DestinationHeight As Integer) As Integer
        <PreserveSig()> Function get_DestinationHeight(ByRef pDestinationHeight As Integer) As Integer
        <PreserveSig()> Function SetSourcePosition(ByVal left As Integer, ByVal top As Integer, ByVal width As Integer, ByVal height As Integer) As Integer
        <PreserveSig()> Function GetSourcePosition(ByRef left As Integer, ByRef top As Integer, ByRef width As Integer, ByRef height As Integer) As Integer
        <PreserveSig()> Function SetDefaultSourcePosition() As Integer
        <PreserveSig()> Function SetDestinationPosition(ByVal left As Integer, ByVal top As Integer, ByVal width As Integer, ByVal height As Integer) As Integer
        <PreserveSig()> Function GetDestinationPosition(ByRef left As Integer, ByRef top As Integer, ByRef width As Integer, ByRef height As Integer) As Integer
        <PreserveSig()> Function SetDefaultDestinationPosition() As Integer
        <PreserveSig()> Function GetVideoSize(ByRef pWidth As Integer, ByRef pHeight As Integer) As Integer
        <PreserveSig()> Function GetVideoPaletteEntries(ByVal StartIndex As Integer, ByVal Entries As Integer, ByRef pRetrieved As Integer, ByVal pPalette As IntPtr) As Integer
        <PreserveSig()> Function GetCurrentImage(ByRef pBufferSize As Integer, ByVal pDIBImage As IntPtr) As Integer
        <PreserveSig()> Function IsUsingDefaultSource() As Integer
        <PreserveSig()> Function IsUsingDefaultDestination() As Integer
        <PreserveSig()> Function GetPreferredAspectRatio(ByRef plAspectX As Integer, ByRef plAspectY As Integer) As Integer
    End Interface

    Public Enum DsEvCode
        None
        Complete = &H1
        UserAbort = &H2
        ErrorAbort = &H3
        Time = &H4
        Repaint = &H5
        StErrStopped = &H6
        StErrStPlaying = &H7
        ErrorStPlaying = &H8
        PaletteChanged = &H9
        VideoSizeChanged = &HA
        QualityChange = &HB
        ShuttingDown = &HC
        ClockChanged = &HD
        Paused = &HE
        OpeningFile = &H10
        BufferingData = &H11
        FullScreenLost = &H12
        Activate = &H13
        NeedRestart = &H14
        WindowDestroyed = &H15
        DisplayChanged = &H16
        Starvation = &H17
        OleEvent = &H18
        NotifyWindow = &H19
        DvdDomChange = &H101
        DvdTitleChange = &H102
        DvdChaptStart = &H103
        DvdAudioStChange = &H104
        DvdSubPicStChange = &H105
        DvdAngleChange = &H106
        DvdButtonChange = &H107
        DvdValidUopsChange = &H108
        DvdStillOn = &H109
        DvdStillOff = &H10A
        DvdCurrentTime = &H10B
        DvdError = &H10C
        DvdWarning = &H10D
        DvdChaptAutoStop = &H10E
        DvdNoFpPgc = &H10F
        DvdPlaybRateChange = &H110
        DvdParentalLChange = &H111
        DvdPlaybStopped = &H112
        DvdAnglesAvail = &H113
        DvdPeriodAStop = &H114
        DvdButtonAActivated = &H115
        DvdCmdStart = &H116
        DvdCmdEnd = &H117
        DvdDiscEjected = &H118
        DvdDiscInserted = &H119
        DvdCurrentHmsfTime = &H11A
        DvdKaraokeMode = &H11B
    End Enum

    <Flags(), ComVisible(False)> _
    Public Enum SeekingCapabilities
        CanSeekAbsolute = &H1
        CanSeekForwards = &H2
        CanSeekBackwards = &H4
        CanGetCurrentPos = &H8
        CanGetStopPos = &H10
        CanGetDuration = &H20
        CanPlayBackwards = &H40
        CanDoSegments = &H80
        Source = &H100
    End Enum

    <StructLayout(LayoutKind.Sequential), ComVisible(False)> _
    Public Class DsOptInt64
        Public Value As Long

        Public Sub New(ByVal Value As Long)
            Me.Value = Value
        End Sub
    End Class

    <Flags(), ComVisible(False)> _
    Public Enum SeekingFlags
        NoPositioning = &H0
        AbsolutePositioning = &H1
        RelativePositioning = &H2
        IncrementalPositioning = &H3
        PositioningBitsMask = &H3
        SeekToKeyFrame = &H4
        ReturnTime = &H8
        Segment = &H10
        NoFlush = &H20
    End Enum

    <ComVisible(False)> _
    Public Enum PinDirection
        Input
        Output
    End Enum
End Namespace
