Imports System.Drawing.Design
Imports System.ComponentModel
Imports System.Collections.Specialized

Public Class ExtendedToolBarButton
    Inherits ToolBarButton

    Private mMenuItem As MenuItem

    <Category("_Extended Properies")> _
    Public Property MenuItem() As MenuItem
        Get
            Return mMenuItem
        End Get
        Set(ByVal Value As MenuItem)
            mMenuItem = Value
        End Set
    End Property
End Class

Public Class ExtendedToolBar
    Inherits ToolBar

    Protected Overrides Sub OnButtonClick(ByVal e As ToolBarButtonClickEventArgs)
        MyBase.OnButtonClick(e)
        CType(e.Button, ExtendedToolBarButton).MenuItem.PerformClick()
    End Sub
End Class

Public Class ExtendedMenuItem
    Inherits MenuItem

    Private Shared mItems As ArrayList

    Private mControl As Control
    Private mObject As Object
    Private mEventHandler As EventHandler
    Private mTag As Object

    Private Const mSEPERATOR_HEIGHT As Integer = 8
    Private Const mSBORDER_WIDTH As Integer = 1
    Private Const mBORDER_SIZE As Integer = mSBORDER_WIDTH * 2
    Private Const mSBUFFER_WIDTH As Integer = 5
    Private Const mLBUFFER_WIDTH As Integer = 15
    Private Const mSHORTCUT_BUFFER_SIZE As Integer = 20
    Private Const mARROW_WIDTH As Integer = 15
    Private mIMAGE_WIDTH As Integer = SystemInformation.SmallIconSize.Width
    Private mIMAGE_HEIGHT As Integer = SystemInformation.SmallIconSize.Height
    Private mIMAGE_BUFFER_SIZE As Integer = SystemInformation.SmallIconSize.Width + 10

    Private mImageList As ImageList
    Private mGraphics As Graphics
    Private mIndex As Integer = -1

    Public Sub New(ByVal text As String)
        MyBase.New()
        Me.Text = text
    End Sub

    Public Sub New(ByVal text As String, ByVal checkStyle As MenuCheckStyle, ByVal eventHandler As System.EventHandler)
        MyBase.New()
        Me.Text = text
        Me.CheckStyle = checkStyle
        Me.OwnerDraw = True
        mEventHandler = eventHandler
    End Sub

    Public Sub New(ByVal text As String, ByVal imageIndex As Integer, ByVal eventHandler As System.EventHandler)
        MyBase.New()
        Me.Text = text
        Me.ImageIndex = imageIndex
        mEventHandler = eventHandler
        Me.OwnerDraw = True
    End Sub

    Private Sub InitImageList()
        If mImageList Is Nothing Then
            Dim mme As ExtendedMainMenu = CType(GetMainMenu(), ExtendedMainMenu)
            If Not mme Is Nothing Then
                mImageList = mme.ImageList
            Else
                mImageList = CType(GetContextMenu(), ExtendedContextMenu).ImageList
            End If
        End If
    End Sub

    Public Function CalcHeight() As Integer
        If Text = "-" Then
            Return mSEPERATOR_HEIGHT
        Else
            If SystemInformation.MenuFont.Height > SystemInformation.SmallIconSize.Height Then
                Return SystemInformation.MenuFont.Height + mBORDER_SIZE
            Else
                Return SystemInformation.SmallIconSize.Height + mBORDER_SIZE
            End If
        End If
    End Function

    Public Function CalcWidth() As Integer
        Dim sf As StringFormat = New StringFormat
        sf.HotkeyPrefix = System.Drawing.Text.HotkeyPrefix.Show
        Dim menuWidth As Integer = CType(mGraphics.MeasureString(Text, SystemInformation.MenuFont, 1000, sf).Width, Integer)
        Dim shortcutWidth As Integer = CType(mGraphics.MeasureString(GetShortcutText, SystemInformation.MenuFont, 1000, sf).Width, Integer)
        If IsTopLevel() = True Then
            Return menuWidth
        Else
            Return mIMAGE_BUFFER_SIZE + menuWidth + mSHORTCUT_BUFFER_SIZE + shortcutWidth
        End If
    End Function

    Public Function HasShortcut() As Boolean
        Return ShowShortcut AndAlso KeyCombination.Key <> Keys.None
    End Function

    Public Function IsSeperator() As Boolean
        Return Text = "-"
    End Function

    Public Function GetShortcutText() As String
        If HasShortcut() Then
            Return KeyCombination.ToString()
        Else
            Return ""
        End If
    End Function

    Private Function IsTopLevel() As Boolean
        Return TypeOf Parent Is MainMenu
    End Function

    Public Sub DrawMenu(ByVal bounds As Rectangle, ByVal selected As Boolean, ByVal index As Integer)
        InitImageList()
        DrawMenuText(bounds, selected)
        If Checked Then
            DrawCheckBox(bounds)
        ElseIf index > -1 Then
            DrawImage(mImageList.Images(index), bounds)
        End If
    End Sub

    Public Sub DrawBackground(ByVal b As Rectangle, ByVal selected As Boolean)
        If selected = True Then
            Dim r As New Rectangle(b.X + 24, b.Y, b.Width - 24, b.Height)
            mGraphics.FillRectangle(SystemBrushes.Highlight, r)
        Else
            mGraphics.FillRectangle(SystemBrushes.Menu, b)
        End If
    End Sub

    Public Sub DrawSeperator(ByVal bounds As Rectangle)
        Dim pen As Pen = New Pen(SystemColors.ControlDark)
        Dim xLeft As Integer = bounds.Left + mIMAGE_BUFFER_SIZE
        Dim xRight As Integer = xLeft + bounds.Width
        Dim yCenter As Integer = bounds.Top + CInt(bounds.Height / 2)
        mGraphics.DrawLine(pen, xLeft, yCenter, xRight, yCenter)
    End Sub

    Private Sub DrawMenuText(ByVal bounds As Rectangle, ByVal selected As Boolean)
        Dim menuFont As Font = SystemInformation.MenuFont
        Dim menuBrush As SolidBrush = Nothing
        If Enabled = False Then
            menuBrush = New SolidBrush(SystemColors.GrayText)
        Else
            If selected = True Then
                menuBrush = New SolidBrush(SystemColors.HighlightText)
            Else
                menuBrush = New SolidBrush(SystemColors.MenuText)
            End If
        End If
        Dim sfMenu As StringFormat = New StringFormat
        sfMenu.HotkeyPrefix = System.Drawing.Text.HotkeyPrefix.Show
        mGraphics.DrawString(Text, menuFont, menuBrush, bounds.Left + mIMAGE_BUFFER_SIZE, bounds.Top + CInt((bounds.Height - menuFont.Height) / 2), sfMenu)
        If HasShortcut() Then
            Dim s As String = GetShortcutText()
            Dim sfShortcut As StringFormat = New StringFormat
            sfShortcut.HotkeyPrefix = System.Drawing.Text.HotkeyPrefix.Show
            mGraphics.DrawString(GetShortcutText, menuFont, menuBrush, bounds.Width - mLBUFFER_WIDTH - s.Length * 5, bounds.Top + CSng((bounds.Height - menuFont.Height) / 2), sfShortcut)
        End If
    End Sub

    Private Sub DrawCheckBox(ByVal b As Rectangle)
        If CheckStyle = CheckStyle.Normal Then
            Dim State As ButtonState = ButtonState.Flat
            If Checked = True Then State = State Or ButtonState.Checked
            If Not Enabled Then State = State Or ButtonState.Inactive
            ControlPaint.DrawCheckBox(mGraphics, b.Left + mSBORDER_WIDTH, b.Top + CInt((b.Height - mIMAGE_HEIGHT) / 2), mIMAGE_WIDTH, mIMAGE_HEIGHT, State)
        ElseIf CheckStyle = CheckStyle.Radio Then
            Dim Left As Integer = b.X + 9
            Dim Top As Integer = b.Y + 6
            Dim r1 As New Rectangle(Left + 1, Top + 0, 3, 5)
            Dim r2 As New Rectangle(Left + 0, Top + 1, 5, 3)
            mGraphics.FillRectangles(New SolidBrush(Color.Black), New Rectangle() {r1, r2})
        End If
    End Sub

    Private Sub DrawImage(ByVal menuImage As Image, ByVal bounds As Rectangle)
        If Enabled = True Then
            mGraphics.DrawImage(menuImage, bounds.Left + mSBORDER_WIDTH, bounds.Top + CInt((bounds.Height - mIMAGE_HEIGHT) / 2), mIMAGE_WIDTH, mIMAGE_HEIGHT)
        Else
            ControlPaint.DrawImageDisabled(mGraphics, menuImage, bounds.Left + mSBORDER_WIDTH, bounds.Top + CInt((bounds.Height - mIMAGE_HEIGHT) / 2), SystemColors.Menu)
        End If
    End Sub

    Protected Overrides Sub OnMeasureItem(ByVal e As MeasureItemEventArgs)
        mGraphics = e.Graphics
        e.ItemHeight = CalcHeight()
        e.ItemWidth = CalcWidth()
    End Sub

    Protected Overrides Sub OnDrawItem(ByVal e As DrawItemEventArgs)
        mGraphics = e.Graphics
        Dim menuSelected As Boolean = (e.State And DrawItemState.Selected) > 0
        DrawBackground(e.Bounds, menuSelected)
        If IsSeperator() = True Then
            DrawSeperator(e.Bounds)
        Else
            DrawMenu(e.Bounds, menuSelected, ImageIndex)
        End If
    End Sub

    <Category("_Extended Properties"), _
     DefaultValue(-1), _
     TypeConverter(GetType(ImageIndexConverter)), _
     Editor("System.Windows.Forms.Design.ImageIndexEditor", GetType(UITypeEditor))> _
    Public Property ImageIndex() As Integer
        Get
            Return mIndex
        End Get
        Set(ByVal Value As Integer)
            mIndex = Value
        End Set
    End Property

    Protected Overrides Sub OnClick(ByVal e As EventArgs)
        If Not mControl Is Nothing Then mControl.Text = CStr(mObject)
        Select Case _CheckStyle
            Case CheckStyle.Normal
                Checked = Not Checked
            Case CheckStyle.Radio
                Dim mi As MenuItem
                For Each mi In Me.Parent.MenuItems
                    Dim mie As ExtendedMenuItem = CType(mi, ExtendedMenuItem)
                    If mie.CheckStyle = CheckStyle.Radio Then
                        mi.Checked = False
                    End If
                Next
                Me.Checked = True
        End Select
        If Not mEventHandler Is Nothing Then mEventHandler.Invoke(Me, e)
        MyBase.OnClick(e)
    End Sub

    Private _CheckStyle As MenuCheckStyle = MenuCheckStyle.None

    'enum as default need workaround, 1.1 only VB
    <Category("_Extended Properties"), _
     DefaultValue(GetType(MenuCheckStyle), "None")> _
    Public Property CheckStyle() As MenuCheckStyle
        Get
            Return _CheckStyle
        End Get
        Set(ByVal Value As MenuCheckStyle)
            _CheckStyle = Value
            RadioCheck = (_CheckStyle = CheckStyle.Radio)
        End Set
    End Property

    Private _handleKey As Boolean = True

    <Category("_Extended Properties"), DefaultValue(True)> _
    Public Property HandleKey() As Boolean
        Get
            Return _handleKey
        End Get
        Set(ByVal Value As Boolean)
            _handleKey = Value
        End Set
    End Property

    Private _canEdit As Boolean = True
    <Category("_Extended Properties"), DefaultValue(True)> _
    Public Property CanEdit() As Boolean
        Get
            Return _canEdit
        End Get
        Set(ByVal Value As Boolean)
            _canEdit = Value
        End Set
    End Property

    Public Function GetIndexOfCheckedRadioSubItem() As Integer
        For Each mi As MenuItem In MenuItems
            If mi.Checked AndAlso mi.RadioCheck Then
                Return mi.Index
            End If
        Next
    End Function

    Public Enum MenuCheckStyle
        None
        Normal
        Radio
    End Enum

    Public Sub New()
        MyBase.New()
    End Sub

    Public Sub Init(ByVal f As Form)
        If HandleKey AndAlso KeyCombination.Key <> Keys.None Then
            AddHandler f.KeyDown, AddressOf f_KeyDown
        End If
    End Sub

    Public Shared Sub Init(ByVal f As Form, ByVal m As Menu, ByVal settings As ArrayList)
        Dim k As KeyCombination

        For Each i As ExtendedMenuItem In ExtendedMenuItem.GetItems(m)
            i.EnglishText = i.Text
            For Each i2 As KeyCombination In settings
                If i.GetID() = i2.Name Then
                    i.KeyCombination = i2
                End If
            Next
            i.Init(f)
        Next
    End Sub

    Public Function GetID() As String
        Dim key As String
        Dim emi As ExtendedMenuItem = Me
        Do
            key += emi.EnglishText
            If TypeOf emi.Parent Is ExtendedMenuItem Then
                emi = CType(emi.Parent, ExtendedMenuItem)
            Else
                Exit Do
            End If
        Loop

        Return key
    End Function

    Public Function GetPath() As String
        Dim key As String
        Dim emi As ExtendedMenuItem = Me
        Do
            key = emi.Text + "/" + key
            If TypeOf emi.Parent Is ExtendedMenuItem Then
                emi = CType(emi.Parent, ExtendedMenuItem)
            Else
                Exit Do
            End If
        Loop

        Return key.Trim("/"c)
    End Function

    Private _englishText As String

    Public Property EnglishText() As String
        Get
            Return _englishText
        End Get
        Set(ByVal Value As String)
            _englishText = Value
        End Set
    End Property

    Private Sub f_KeyDown(ByVal sender As Object, ByVal e As KeyEventArgs)
        If Enabled AndAlso CType(e.KeyData, Keys) = (KeyCombination.Modifier Or KeyCombination.Key) Then
            PerformClick()
        End If
    End Sub

    Public Shared Function GetItems(ByVal menu As Menu) As ExtendedMenuItem()
        mItems = New ArrayList
        AddMenuItems(menu)
        Return CType(mItems.ToArray(GetType(ExtendedMenuItem)), ExtendedMenuItem())
    End Function

    Private Shared Sub AddMenuItems(ByVal m As Menu)
        Dim mi As Windows.Forms.MenuItem

        For Each mi In m.MenuItems
            If TypeOf mi Is ExtendedMenuItem Then
                mItems.Add(mi)
            End If
            AddMenuItems(mi)
        Next
    End Sub

    Private _keyCombination As KeyCombination

    <Category("_Extended Properties"), _
     DefaultValue(GetType(KeyCombination), "None+None")> _
    Public Property KeyCombination() As KeyCombination
        Get
            Return _keyCombination
        End Get
        Set(ByVal Value As KeyCombination)
            _keyCombination = Value
        End Set
    End Property
End Class

Public Class ExtendedListBox
    Inherits ListBox

    Public ReadOnly Property SelectedListItem() As ListItem
        Get
            Try
                Return CType(SelectedItem, ListItem)
            Catch
            End Try
        End Get
    End Property

    Public Function GetListItemArray() As ListItem()
        Dim listItems(Items.Count - 1) As ListItem
        Items.CopyTo(listItems, 0)
        Return listItems
    End Function

    Public Function GetObjectArray() As Object()
        Dim listItems(Items.Count - 1) As Object
        Items.CopyTo(listItems, 0)
        Return listItems
    End Function

    Public Function GetArrayList() As ArrayList
        Dim items As New ArrayList
        items.AddRange(GetObjectArray)
        Return items
    End Function

    Public Sub MoveSelectedItemUp()
        Dim i As Integer = SelectedIndex
        If i > 0 Then
            Items.Insert(i - 1, SelectedItem)
            Items.RemoveAt(i + 1)
            SelectedIndex = i - 1
        End If
    End Sub

    Public Sub MoveSelectedItemDown()
        Dim i As Integer = SelectedIndex
        If i < Items.Count - 1 Then
            Items.Insert(i + 2, SelectedItem)
            Items.RemoveAt(i)
            SelectedIndex = i + 1
        End If
    End Sub

    Public Sub DeleteSelectedItem()
        Try 'strange
            Dim i As Integer = SelectedIndex
            Items.Remove(SelectedItem)
            If Items.Count > i Then
                SelectedIndex = i
            Else
                If Items.Count > 0 Then SelectedIndex = Items.Count - 1
            End If
        Catch
        End Try
    End Sub

    Public Sub Add(ByVal item As Object)
        Items.Add(item)
        SelectedItem = item
    End Sub

    Public Sub Reload()
        Dim sel As Object = SelectedItem
        Dim a As Object() = GetObjectArray()
        Items.Clear()
        Items.AddRange(a)
        SelectedItem = sel
    End Sub
End Class

Public Class ExtendedComboBox
    'Double click event don't work in .NET 1.1.
    Inherits ComboBox

    Protected Overrides Sub OnClick(ByVal e As EventArgs)
        Static LastTickCount As Long
        Dim TickCount As Long = Environment.TickCount
        If TickCount - LastTickCount < 300 Then
            OnDoubleClick(e)
            SelectionStart = 0
            SelectionLength = Text.Length
        End If
        LastTickCount = TickCount
        MyBase.OnClick(e)
    End Sub
End Class

Public Class ExtendedContextMenu
    Inherits ContextMenu

    Private mImageList As ImageList

    <Category("_Extended Properies")> _
    Public Property ImageList() As ImageList
        Get
            Return mImageList
        End Get
        Set(ByVal Value As ImageList)
            mImageList = Value
        End Set
    End Property

    Public Function GetIndexOfCheckedRadioSubItem() As Integer
        For Each mi As MenuItem In MenuItems
            If mi.Checked AndAlso mi.RadioCheck Then
                Return mi.Index
            End If
        Next
    End Function
End Class

Public Class ExtendedMainMenu
    Inherits MainMenu

    Private mImageList As ImageList

    <Category("_Extended Properies")> _
    Public Property ImageList() As ImageList
        Get
            Return mImageList
        End Get
        Set(ByVal Value As ImageList)
            mImageList = Value
        End Set
    End Property

    Public ReadOnly Property MenuItemsAll() As MenuItem()
        Get
            Return ExtendedMenuItem.GetItems(Me)
        End Get
    End Property
End Class

<Serializable()> _
Public Class ExtendedStringCollection
    Inherits StringCollection

    Public Function ToStringArray() As String()
        Dim sa As String() = New String(Count - 1) {}
        CopyTo(sa, 0)
        Return sa
    End Function

    Public Sub SetItems(ByVal items As String())
        Clear()
        AddRange(items)
    End Sub
End Class

Public Class ExtendedTrackBar
    Inherits TrackBar

    Protected Overrides Sub OnMouseWheel(ByVal e As MouseEventArgs)
        If Orientation = Orientation.Horizontal Then
            Dim Val As Integer = Value
            If e.Delta > 0 Then
                Val += 2
                If Val > Maximum Then Val = Maximum
            Else
                Val -= 2
                If Val < Minimum Then Val = Minimum
            End If
            Value = Val
        End If
        OnScroll(Nothing)
        OnValueChanged(Nothing)
    End Sub
End Class