Imports System.ComponentModel
Imports System.ComponentModel.Design.Serialization
Imports System.Reflection
Imports System.Globalization

<Serializable(), TypeConverter(GetType(KeyCombination.KeyCombinationTypeConverter))> _
Public Structure KeyCombination
    Implements IComparable

    Public Sub New(ByVal name As String, ByVal key As Keys)
        _name = name
        _key = key
    End Sub

    Public Sub New(ByVal name As String, ByVal modifier As Keys, ByVal key As Keys)
        _name = name
        _modifier = modifier
        _key = key
    End Sub

    Private _name As String

    <Browsable(False)> _
    Public Property Name() As String
        Get
            Return _name
        End Get
        Set(ByVal Value As String)
            _name = Value
        End Set
    End Property

    Private _modifier As Keys

    'enum as default need workaround, 1.1 only VB
    <DefaultValue(GetType(Keys), "None")> _
    Public Property Modifier() As Keys
        Get
            Return _modifier
        End Get
        Set(ByVal Value As Keys)
            _modifier = Value
        End Set
    End Property

    Private _key As Keys

    'enum as default need workaround, 1.1 only VB
    <DefaultValue(GetType(Keys), "None")> _
    Public Property Key() As Keys
        Get
            Return _key
        End Get
        Set(ByVal Value As Keys)
            _key = Value
        End Set
    End Property

    Public Function CompareTo(ByVal obj As Object) As Integer Implements IComparable.CompareTo
        Return Name.CompareTo(CType(obj, KeyCombination).Name)
    End Function

    Private Function GetFriendlyKey(ByVal k As Keys) As String
        Dim s As String = k.ToString()

        If s.Length > 1 Then
            Select Case s
                Case "Control"
                    s = g.GetResource("Ctrl")
                Case "Shift"
                    s = g.GetResource("Shift")
                Case "OemQuestion"
                    s = "#"
                Case "Space"
                    s = g.GetResource("Space")
                Case "Enter"
                    s = g.GetResource("Enter")
                Case "None"
                    s = g.GetResource("None")
            End Select
        End If

        Return s
    End Function

    Public Overrides Function ToString() As String
        If Modifier = Keys.None Then
            Return GetFriendlyKey(Key)
        Else
            Return GetFriendlyKey(Modifier) + "+" + GetFriendlyKey(Key)
        End If
    End Function

    Public Enum Keys
        None = 0
        LButton = 1
        RButton = 2
        Cancel = 3
        MButton = 4
        XButton1 = 5
        XButton2 = 6
        Back = 8
        Tab = 9
        LineFeed = 10
        Clear = 12
        Enter = 13
        ShiftKey = 16
        ControlKey = 17
        Menu = 18
        Pause = 19
        Capital = 20
        HangulMode = 21
        JunjaMode = 23
        FinalMode = 24
        HanjaMode = 25
        Escape = 27
        IMEConvert = 28
        IMENonconvert = 29
        IMEAceept = 30
        IMEModeChange = 31
        Space = 32
        Prior = 33
        [Next] = 34
        [End] = 35
        Home = 36
        Left = 37
        Up = 38
        Right = 39
        Down = 40
        [Select] = 41
        Print = 42
        Execute = 43
        Snapshot = 44
        Insert = 45
        Delete = 46
        Help = 47
        D0 = 48
        D1 = 49
        D2 = 50
        D3 = 51
        D4 = 52
        D5 = 53
        D6 = 54
        D7 = 55
        D8 = 56
        D9 = 57
        A = 65
        B = 66
        C = 67
        D = 68
        E = 69
        F = 70
        G = 71
        H = 72
        I = 73
        J = 74
        K = 75
        L = 76
        M = 77
        N = 78
        O = 79
        P = 80
        Q = 81
        R = 82
        S = 83
        T = 84
        U = 85
        V = 86
        W = 87
        X = 88
        Y = 89
        Z = 90
        LWin = 91
        RWin = 92
        Apps = 93
        NumPad0 = 96
        NumPad1 = 97
        NumPad2 = 98
        NumPad3 = 99
        NumPad4 = 100
        NumPad5 = 101
        NumPad6 = 102
        NumPad7 = 103
        NumPad8 = 104
        NumPad9 = 105
        Multiply = 106
        Add = 107
        Separator = 108
        Subtract = 109
        [Decimal] = 110
        Divide = 111
        F1 = 112
        F2 = 113
        F3 = 114
        F4 = 115
        F5 = 116
        F6 = 117
        F7 = 118
        F8 = 119
        F9 = 120
        F10 = 121
        F11 = 122
        F12 = 123
        F13 = 124
        F14 = 125
        F15 = 126
        F16 = 127
        F17 = 128
        F18 = 129
        F19 = 130
        F20 = 131
        F21 = 132
        F22 = 133
        F23 = 134
        F24 = 135
        NumLock = 144
        Scroll = 145
        LShiftKey = 160
        RShiftKey = 161
        LControlKey = 162
        RControlKey = 163
        LMenu = 164
        RMenu = 165
        BrowserBack = 166
        BrowserForward = 167
        BrowserRefresh = 168
        BrowserStop = 169
        BrowserSearch = 170
        BrowserFavorites = 171
        BrowserHome = 172
        VolumeMute = 173
        VolumeDown = 174
        VolumeUp = 175
        MediaNextTrack = 176
        MediaPreviousTrack = 177
        MediaStop = 178
        MediaPlayPause = 179
        LaunchMail = 180
        SelectMedia = 181
        LaunchApplication1 = 182
        LaunchApplication2 = 183
        OemSemicolon = 186
        Oemplus = 187
        Oemcomma = 188
        OemMinus = 189
        OemPeriod = 190
        OemQuestion = 191
        Oemtilde = 192
        OemOpenBrackets = 219
        OemPipe = 220
        OemCloseBrackets = 221
        OemQuotes = 222
        Oem8 = 223
        OemBackslash = 226
        ProcessKey = 229
        Attn = 246
        Crsel = 247
        Exsel = 248
        EraseEof = 249
        Play = 250
        Zoom = 251
        NoName = 252
        Pa1 = 253
        OemClear = 254
        KeyCode = 65535
        Shift = 65536
        Control = 131072
        Alt = 262144
        Modifiers = -65536
    End Enum

    Public Class KeyCombinationTypeConverter
        Inherits ExpandableObjectConverter

        Public Overloads Overrides Function CanConvertTo(ByVal context As ITypeDescriptorContext, ByVal destinationType As Type) As Boolean
            If destinationType Is GetType(InstanceDescriptor) Then
                Return True
            End If
            Return MyBase.CanConvertTo(context, destinationType)
        End Function

        Public Overloads Overrides Function ConvertTo(ByVal context As ITypeDescriptorContext, ByVal culture As CultureInfo, ByVal value As Object, ByVal destinationType As Type) As Object
            Dim k As KeyCombination = CType(value, KeyCombination)

            If destinationType Is GetType(InstanceDescriptor) Then
                Dim ci As ConstructorInfo = GetType(KeyCombination).GetConstructor(New Type() {GetType(System.String), GetType(KeyCombination.Keys), GetType(KeyCombination.Keys)})
                Return New InstanceDescriptor(ci, New Object() {k.Name, k.Modifier, k.Key})
            ElseIf destinationType Is GetType(String) Then
                Return k.Modifier.ToString() + "+" + k.Key.ToString()
            End If

            Return MyBase.ConvertTo(context, culture, value, destinationType)
        End Function

        Public Overloads Overrides Function CanConvertFrom(ByVal context As System.ComponentModel.ITypeDescriptorContext, ByVal sourceType As System.Type) As Boolean
            If sourceType Is GetType(String) Then
                Return True
            Else
                Return MyBase.CanConvertFrom(context, sourceType)
            End If
        End Function

        Public Overloads Overrides Function ConvertFrom(ByVal context As ITypeDescriptorContext, ByVal culture As System.Globalization.CultureInfo, ByVal val As Object) As Object
            If TypeOf val Is String Then
                Dim a As String() = CStr(val).Split("+"c)
                Return New KeyCombination(Nothing, CType(TypeDescriptor.GetConverter(GetType(KeyCombination.Keys)).ConvertFromString(a(0)), KeyCombination.Keys), CType(TypeDescriptor.GetConverter(GetType(KeyCombination.Keys)).ConvertFromString(a(1)), KeyCombination.Keys))
            End If
            Return MyBase.ConvertFrom(context, culture, val)
        End Function
    End Class
End Structure