Imports System
Imports System.Windows.Forms
Imports System.IO
Imports System.Collections
Imports System.Collections.Specialized
Imports System.Runtime.Serialization
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.Reflection

<Serializable()> _
Public Class Settings
    Implements ISerializable

    Public Versions As Hashtable = New Hashtable

    <Version(111)> Public Language As String = "English"
    <Version(111)> Public Font As Font = New Font(New FontFamily("Courier New"), 10, FontStyle.Regular)
    <Version(111)> Public FilterView As Integer
    <Version(111)> Public Profiles As Object() = New Object() {}
    <Version(111)> Public HidePreviewButtons As Boolean
    <Version(111)> Public RecentFiles As New StringCollection
    <Version(111)> Public SourceCodeDirectory As String = ""
    <Version(111)> Public ExternalPreview As String = Paths.GetProgramsDir + "Windows Media Player\mplayer2.exe"
    <Version(111)> Public FilterCategories As String() = {"Producing", "Processing", "Editing", "Interlaced", "Audio", "Special", "Debug", "Resize", "Noise", "Source", "Sharpen"}
    <Version(117)> Public KeyCombinations As New ArrayList
    <Version(111)> Public LastOverloadedFilter As New Hashtable

    Public Sub New()
    End Sub

    Public Sub Examine()
        Dim s As Settings = Nothing
        For Each fi As FieldInfo In Me.GetType().GetFields()
            If Not (fi.Name = "Versions") Then
                Dim version As Integer = CType(fi.GetCustomAttributes(False)(0), VersionAttribute).Value
                If Not Versions.ContainsKey(fi.Name) OrElse Not (CInt(Versions(fi.Name)) = version) Then
                    If s Is Nothing Then
                        s = New Settings
                        s.Init()
                    End If
                    fi.SetValue(Me, s.GetType().GetField(fi.Name).GetValue(s))
                    Versions(fi.Name) = version
                End If
            End If
        Next
    End Sub

    Public Sub Init()

    End Sub

    Public Shared Sub Load(ByRef s As Settings, ByVal path As String)
        Try
            s = CType(Deserialize(path), Settings)
        Catch
            If File.Exists(path) Then
                Msg.Show("The settings couldn't be loaded.", "Error", MessageBoxIcon.Error)
            End If
        End Try
        s.Examine()
    End Sub

    Public Shared Sub Serialize(ByVal obj As Object, ByVal path As String)
        Dim fs As FileStream = New FileStream(path, FileMode.OpenOrCreate)
        Dim bf As BinaryFormatter = New BinaryFormatter
        bf.Serialize(fs, obj)
        fs.Close()
    End Sub

    Public Shared Function Deserialize(ByVal path As String) As Object
        Dim fs As FileStream = New FileStream(path, FileMode.Open)
        Dim bf As BinaryFormatter = New BinaryFormatter
        Return bf.Deserialize(fs)
        fs.Close()
    End Function

    <AttributeUsage(AttributeTargets.Field)> _
    Class VersionAttribute
        Inherits Attribute

        Public Value As Integer

        Public Sub New(ByVal version As Integer)
            Value = version
        End Sub
    End Class

    Public Sub New(ByVal info As SerializationInfo, ByVal context As StreamingContext)
        For Each fi As FieldInfo In Me.GetType().GetFields()
            Try
                fi.SetValue(Me, info.GetValue(fi.Name, fi.FieldType))
            Catch
            End Try
        Next
    End Sub

    Public Overridable Sub GetObjectData(ByVal info As SerializationInfo, ByVal context As StreamingContext) Implements ISerializable.GetObjectData
        For Each fi As FieldInfo In Me.GetType().GetFields()
            Try
                info.AddValue(fi.Name, fi.GetValue(Me))
            Catch
            End Try
        Next
    End Sub
End Class