Public Class Translator
    Public TableEnglish As New Hashtable
    Public TableTranslation As New Hashtable
    Private mChanged As String

    Public Function GetEnglish(ByVal key As String) As String
        Dim st As String = CStr(TableEnglish(key))
        If st Is Nothing Then Return ""
        Return st
    End Function

    Public Function GetTranslation(ByVal key As String) As String
        Dim st As String = CStr(TableTranslation(key))
        If st Is Nothing Then Return ""
        Return st
    End Function

    Public Sub Save()
        Dim s1 As New Str
        Dim s2 As New Str
        Dim st, Val As String
        Dim sc As New ExtendedStringCollection
        Dim sa As String()
        For Each st In TableEnglish.Keys
            Val = PrepareToSave(GetEnglish(st))
            If Val <> "" AndAlso st <> "" Then sc.Add(st + " = " + Val)
        Next
        sa = sc.ToStringArray
        sa.Sort(sa)
        For Each st In sa
            s1.Add(st)
        Next
        sc.Clear()
        For Each st In TableTranslation.Keys
            If TableEnglish.ContainsKey(st) Then
                Val = PrepareToSave(GetTranslation(st))
                If Val <> "" AndAlso st <> "" Then sc.Add(st + " = " + Val)
            End If
        Next
        sa = sc.ToStringArray
        sa.Sort(sa)
        For Each st In sa
            s2.Add(st)
        Next
        s1.Save(Paths.GetApplicationDir + "languages\" + "English.txt", System.Text.Encoding.UTF8)
        If Not g.Settings.Language = "English" Then
            s2.Save(Paths.GetApplicationDir + "languages\" + g.Settings.Language + ".txt", System.Text.Encoding.UTF8)
        End If
    End Sub

    Private Function PrepareToSave(ByVal st As String) As String
        PrepareToSave = Replace(st, vbCrLf, "<NewLine>")
        Return Replace(PrepareToSave, vbCr, "<NewLine>")
    End Function

    Private Function PrepareToLoad(ByVal value As String) As String
        PrepareToLoad = Replace(value, vbCrLf, "")
        Return Replace(PrepareToLoad, "<NewLine>", vbCrLf)
    End Function

    Public Sub Load()
        TableEnglish.Clear()
        TableTranslation.Clear()

        Dim file1 As New Str
        file1.AddFile(Paths.GetApplicationDir + "languages\" + "English.txt")
        For Each line As String In file1.Lines
            Dim key As String = Str.GetLeft(line, " =")
            Dim value As String = PrepareToLoad(Str.GetRight(line, "= "))
            If key <> "" AndAlso value <> "" Then
                TableEnglish(key) = value
            End If
        Next
        If Not g.Settings.Language = "English" Then
            Dim file2 As New Str
            file2.AddFile(Paths.GetApplicationDir + "languages\" + g.Settings.Language + ".txt", System.Text.Encoding.UTF8)
            For Each line As String In file2.Lines
                TableTranslation(Str.GetLeft(line, " =")) = PrepareToLoad(Str.GetRight(line, "= "))
            Next
        End If
    End Sub

    Public Sub SetEnglishPropertiesOnly(ByVal co As Control)
        Dim KeyTooltip As String
        Dim c As Control
        For Each c In co.Controls
            KeyTooltip = CStr(c.Tag)
            If KeyTooltip <> "" AndAlso TableEnglish.ContainsKey(KeyTooltip) Then
                g.Tooltip.SetToolTip(c, GetEnglish(KeyTooltip))
            End If
            SetEnglishPropertiesOnly(c)
        Next
    End Sub

    Public Sub SetBothProperties(ByVal co As Control)
        Dim Text, ControlText, Trimmed, KeyTooltip As String
        Dim ControlType As Type
        Dim c As Control
        For Each c In co.Controls
            ControlText = c.Text
            If ControlText <> "" AndAlso Not Str.Contains(c.Name, "_") Then
                If ControlText.EndsWith(":") Then
                    Text = GetTranslation(ControlText.TrimEnd(":"c))
                    If Text <> "" Then c.Text = Text + ":"
                Else
                    Text = GetTranslation(ControlText)
                    If Text <> "" Then c.Text = Text
                End If
            End If
            KeyTooltip = CStr(c.Tag)
            If KeyTooltip <> "" Then
                Text = GetTranslation(KeyTooltip)
                If Text = "" Then Text = GetEnglish(KeyTooltip)
                If Text <> "" Then g.Tooltip.SetToolTip(c, Text)
            End If
            SetBothProperties(c)
        Next
    End Sub

    Public Sub Translate(ByVal form As Form)
        g.Tooltip = New ToolTip
        g.Tooltip.AutoPopDelay = 20000
        g.Tooltip.InitialDelay = 500
        g.Tooltip.ReshowDelay = 1000

        If g.Settings.Language <> "English" Then
            If Not Str.Contains(form.Text, "_") Then
                Dim Text As String = GetTranslation(form.Text)
                If Text <> "" Then form.Text = Text
            Else
                form.Text = form.Text.Replace("_", "")
            End If
            SetBothProperties(form)
        Else
            SetEnglishPropertiesOnly(form)
        End If
        form.Refresh()
    End Sub

    Public Sub Translate(ByVal toolBar As ToolBar)
        Dim tbb As ToolBarButton

        For Each tbb In toolBar.Buttons
            If Str.Contains(tbb.Text, "_") Then
                tbb.Text = tbb.Text.Replace("_", "")
            ElseIf tbb.Text <> "" Then
                tbb.Text = g.GetResource(tbb.Text)
            End If
            If tbb.ToolTipText <> "" Then
                tbb.ToolTipText = g.GetResource(tbb.ToolTipText)
            End If
        Next
    End Sub

    Public Sub Translate(ByVal menu As Menu)
        Dim mi As MenuItem
        For Each mi In ExtendedMenuItem.GetItems(menu)
            If mi.Text <> "-" Then
                If Str.Contains(mi.Text, "_") Then
                    mi.Text = mi.Text.Replace("_", "")
                Else
                    If mi.Text.EndsWith("...") Then
                        mi.Text = g.GetResource(Str.GetLeft(mi.Text, "...")) + "..."
                    Else
                        mi.Text = g.GetResource(mi.Text)
                    End If
                End If
            End If
        Next
    End Sub

    Public Sub Translate(ByVal listView As ListView)
        For Each i As ColumnHeader In listView.Columns
            i.Text = g.GetResource(i.Text)
        Next
    End Sub

    Public Sub InitChangedState()
        mChanged = GetChangeValue()
    End Sub

    Private Function GetChangeValue() As String
        Dim st As New System.Text.StringBuilder
        For Each i As String In TableEnglish.Keys
            st.Append(i)
            st.Append(CStr(TableEnglish(i)))
        Next
        For Each i As String In TableTranslation.Keys
            st.Append(i)
            st.Append(CStr(TableTranslation(i)))
        Next
        Return st.ToString
    End Function

    Public ReadOnly Property WasModiefied() As Boolean
        Get
            Return mChanged <> GetChangeValue()
        End Get
    End Property

    Public Function GetKeys() As Specialized.StringCollection
        Dim sc As New Specialized.StringCollection
        For Each i As String In TableEnglish.Keys
            sc.Add(i)
        Next
        Return sc
    End Function

    Public Function ContainsKey(ByVal key As String) As Boolean
        Return TableEnglish.ContainsKey(key)
    End Function

    Public Sub SetEnglisch(ByVal key As String, ByVal value As String)
        If key <> "" Then
            TableEnglish(key) = value
        End If
    End Sub

    Public Sub SetTranslation(ByVal key As String, ByVal value As String)
        If key <> "" Then
            TableTranslation(key) = value
        End If
    End Sub

    Public Sub RemoveKey(ByVal key As String)
        TableEnglish.Remove(key)
        TableTranslation.Remove(key)
    End Sub
End Class