Imports System.Runtime.InteropServices

<Serializable()> Public Class ListItem
    Public Text As String
    Public [String] As String
    Public String2 As String
    Public Bool As Boolean
    Public Bool2 As Boolean
    Public Int As Boolean
    Public Int2 As Boolean
    Public [Object] As Boolean
    Public Object2 As Boolean

    Public Sub New(ByVal text As String)
        Me.Text = text
    End Sub

    Public Sub New(ByVal text As String, ByVal bool As Boolean)
        Me.Text = text
        Me.Bool = bool
    End Sub

    Public Sub New(ByVal text As String, ByVal [string] As String)
        Me.Text = text
        Me.String = [string]
    End Sub

    Public Sub New(ByVal text As String, ByVal [string] As String, ByVal string2 As String)
        Me.Text = text
        Me.String = [string]
        Me.String2 = string2
    End Sub

    Public Overrides Function ToString() As String
        Return Text
    End Function
End Class

<Serializable()> _
Public Class Plugin
    Public Name As String
    Public Filters As New FilterCollection
    Public Description As String

    Public Overrides Function ToString() As String
        Return Name
    End Function
End Class

<Serializable()> _
Public Class Filter
    Public Name As String = ""
    Public Helpfile As String
    Public Parameters As New ParameterCollection
    Public Description As String
    Public Category As String
    Public Plugin As Plugin
    Public Type As AviSynthType
    Public Colorspaces As New ExtendedStringCollection
    Public DisplayFullName As Boolean

    Public Overrides Function ToString() As String
        If DisplayFullName Then
            Return Plugin.Name + "::" + Name
        Else
            Return Name
        End If
    End Function
End Class

<Serializable()> _
Public Class Parameter
    Public Name As String = ""
    Public Type As AviSynthType
    Public [Default] As String
    Public Description As String
    Public Min As Double
    Public Max As Double
    Public [Step] As Double
    Public [Optional] As Boolean
    Public ParameterList As Boolean
    Public Items As New ExtendedStringCollection
    Public Value As String

    Public Overrides Function ToString() As String
        Return Name
    End Function
End Class

Public Enum AviSynthType
    clip
    [string]
    int
    bool
    float
    [variant]
End Enum

Public Class WinAPI
    Public Class User32
        <DllImport("user32.dll")> _
        Public Shared Function PostMessage(ByVal handle As IntPtr, ByVal Msg As UInt32, ByVal wParam As Int32, ByVal lParam As Int32) As Boolean
        End Function

        <DllImport("user32", CharSet:=CharSet.Auto)> _
        Public Shared Function SendMessage(ByVal handle As IntPtr, ByVal message As Integer, ByVal wParam As Integer, ByVal lParam As Integer) As Integer
        End Function
    End Class

    Public Class Window
        Public Enum WM
            GETTEXT = 13
            GETTEXTLENGTH = 14
            CLOSE = 16
            KEYDOWN = 256
        End Enum
    End Class
End Class

Public Class Paths
    Public Shared Function GetApplicationDir() As String
        Return Dir.Backslash(Application.StartupPath)
    End Function

    Public Shared Function GetPluginDir() As String
        Return Dir.Backslash(Registry.GetString(Registry.Root.LocalMachine, "SOFTWARE\AviSynth", "plugindir2_5"))
    End Function

    Public Shared Function GetProgramsDir() As String
        Return Dir.Backslash(Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles))
    End Function
End Class

Public Class Registry
    Public Shared Function GetString(ByVal root As Root, ByVal key As String, ByVal name As String) As String
        Try
            If root = root.CurrentUser Then Return CStr(Microsoft.Win32.Registry.CurrentUser.OpenSubKey(key).GetValue(name))
            If root = root.LocalMachine Then Return CStr(Microsoft.Win32.Registry.LocalMachine.OpenSubKey(key).GetValue(name))
            If root = root.ClassesRoot Then Return CStr(Microsoft.Win32.Registry.ClassesRoot.OpenSubKey(key).GetValue(name))
        Catch
            Return ""
        End Try
    End Function

    Public Shared Sub SetDefaultExtension(ByVal extensionNoDot As String)
        Dim value As String = Registry.GetString(Root.ClassesRoot, "." + extensionNoDot, "")
        If value = "" Then
            Registry.Write(Root.ClassesRoot, "." + extensionNoDot, "", extensionNoDot + "file")
            value = extensionNoDot + "file"
        End If
        Registry.Write(Root.ClassesRoot, value + "\shell\" + Application.ProductName + "\command", "", """" & Application.ExecutablePath + """ ""%1""")
        Registry.Write(Root.ClassesRoot, value + "\shell", "", Application.ProductName)
        DeleteKey(Root.ClassesRoot, value + "\DefaultIcon")
        DeleteValue(Root.CurrentUser, "Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\." + extensionNoDot, "Application")
    End Sub

    Public Shared Sub Write(ByVal root As Root, ByVal key As String, ByVal name As String, ByVal obj As Object)
        If root = root.CurrentUser Then Microsoft.Win32.Registry.CurrentUser.CreateSubKey(key).SetValue(name, obj)
        If root = root.LocalMachine Then Microsoft.Win32.Registry.LocalMachine.CreateSubKey(key).SetValue(name, obj)
        If root = root.ClassesRoot Then Microsoft.Win32.Registry.ClassesRoot.CreateSubKey(key).SetValue(name, obj)
    End Sub

    Public Shared Sub WriteApplication(ByVal name As String, ByVal obj As Object)
        Write(Root.LocalMachine, "SOFTWARE\" + Application.ProductName, name, obj)
    End Sub

    Public Shared Function ReadApplicationString(ByVal name As String) As String
        Return GetString(Root.LocalMachine, "SOFTWARE\" + Application.ProductName, name)
    End Function

    Public Shared Sub DeleteValue(ByVal root As Root, ByVal key As String, ByVal name As String)
        Try
            Select Case root
                Case root.LocalMachine
                    Microsoft.Win32.Registry.LocalMachine.OpenSubKey(key, True).DeleteValue(name)
                Case root.CurrentUser
                    Microsoft.Win32.Registry.CurrentUser.OpenSubKey(key, True).DeleteValue(name)
                Case root.ClassesRoot
                    Microsoft.Win32.Registry.ClassesRoot.OpenSubKey(key, True).DeleteValue(name)
            End Select
        Catch
        End Try
    End Sub

    Public Shared Sub DeleteKey(ByVal root As Root, ByVal key As String)
        Try
            If root = root.LocalMachine Then Microsoft.Win32.Registry.LocalMachine.DeleteSubKeyTree(key)
            If root = root.CurrentUser Then Microsoft.Win32.Registry.CurrentUser.DeleteSubKeyTree(key)
            If root = root.ClassesRoot Then Microsoft.Win32.Registry.ClassesRoot.DeleteSubKeyTree(key)
        Catch
        End Try
        Try
            If root = root.LocalMachine Then Microsoft.Win32.Registry.LocalMachine.DeleteSubKey(key)
            If root = root.CurrentUser Then Microsoft.Win32.Registry.CurrentUser.DeleteSubKey(key)
            If root = root.ClassesRoot Then Microsoft.Win32.Registry.ClassesRoot.DeleteSubKey(key)
        Catch
        End Try
    End Sub

    Public Enum Root
        CurrentUser
        LocalMachine
        ClassesRoot
    End Enum
End Class

Public Class Script
    Private Shared Rate As Double
    Private Shared Frames As Integer
    Private Shared Width As Integer
    Private Shared Height As Integer
    Private Shared Compare As String
    Private Shared mScript As String

    Public Shared Function GetRate(ByVal script As String) As Double
        mScript = script
        Infos()
        Return Rate
    End Function

    Public Shared Function GetFrames(ByVal script As String) As Integer
        mScript = script
        Infos()
        Return Frames
    End Function

    Public Shared Function GetWidth(ByVal script As String) As Integer
        mScript = script
        Infos()
        Return Width
    End Function

    Public Shared Function GetHeight(ByVal script As String) As Integer
        mScript = script
        Infos()
        Return Height
    End Function

    Public Shared Sub Infos()
        If mScript <> Compare Then
            Dim s As New Str(mScript)
            s.Save(Paths.GetApplicationDir + "temp\Values.avs")
            Dim m As New DirectShow.VideoPlayer
            m.OpenFile(Paths.GetApplicationDir + "temp\Values.avs")
            Rate = m.GetFramerate
            Frames = m.GetFrames
            Width = m.GetWidth
            Height = m.GetHeight
            m.Close()
            Compare = mScript
        End If
    End Sub
End Class

Public Interface IFilterEditor
    Sub ShowEditor()
    Property Signatur() As EditorSignatur
End Interface

Public Class EditorSignatur
    Public Name As String
    Public Signatur As String()
    Public Editor As Type

    Public Sub New(ByVal name As String, ByVal signatur As String(), ByVal editor As Type)
        Me.Name = name
        Me.Signatur = signatur
        Me.Editor = editor
    End Sub
End Class

Public Class Msg
    Private Shared _text As String
    Private Shared _caption As String
    Private Shared _icon As MessageBoxIcon

    Public Shared Function Show(ByVal text As String, ByVal caption As String, ByVal icon As MessageBoxIcon) As DialogResult
        Return MessageBox.Show(text, caption, MessageBoxButtons.OK, icon)
    End Function

    Public Shared Sub ShowAsync(ByVal text As String, ByVal caption As String, ByVal icon As MessageBoxIcon)
        _text = text
        _caption = caption
        _icon = icon

        Dim t As New Threading.Thread(AddressOf ShowAsync)
        t.Start()
    End Sub

    Private Shared Sub ShowAsync()
        Show(_text, _caption, _icon)
    End Sub
End Class