Imports System.IO
Imports System.Runtime.Serialization.Formatters.Binary

Public Class g
    Public Shared Translator As New Translator
    Public Shared Settings As New Settings
    Public Shared Tooltip As New Tooltip
    Public Shared Filters As New FilterCollection
    Public Shared Plugins As New ArrayList
    Public Shared FilterTreeNodesByPlugin As TreeNode()
    Public Shared FilterTreeNodesAll As TreeNode()
    Public Shared FilterTreeNodesByCategory As TreeNode()
    Public Shared FormMain As fMain
    Public Shared PreviewPosition As Integer
    Private Shared PerformanceTestStart As Long

    Public Shared Function GetResource(ByVal key As String) As String
        Dim Value As String
        If g.Settings.Language = "English" Then
            Value = g.Translator.GetEnglish(key)
        Else
            Value = g.Translator.GetTranslation(key)
            If Value = "" Then Value = g.Translator.GetEnglish(key)
        End If
        If Value = "" Then Value = key
        Return Value
    End Function

    Public Shared Function GetParameterText(ByVal f As Filter) As String
        Dim paramList As New ExtendedStringCollection
        For Each parameter As Parameter In f.Parameters
            Dim paramString As String
            If parameter.Optional Then
                paramString = "optional " + parameter.Type.ToString + " " + parameter.Name
                If parameter.Default <> "" Then
                    paramString += " = " + parameter.Default
                End If
            Else
                paramString = parameter.Type.ToString + " " + parameter.Name
            End If
            paramList.Add(paramString)
        Next
        Return Join(paramList.ToStringArray, vbCrLf)
    End Function

    Public Shared Function GetInteger(ByVal value As String) As Integer
        If Not IsNumeric(value) Then Return 0
        Return CInt(value)
    End Function

    Public Shared Sub ShowHelp(ByVal page As String)
        Dim Path As String = Paths.GetApplicationDir + "help\" + g.Settings.Language + ".chm"
        If Not File.Exists(Path) Then
            Path = Paths.GetApplicationDir + "help\English" + ".chm"
        End If
        If page <> "" Then
            Help.ShowHelp(g.FormMain, Path, HelpNavigator.Topic, page)
        Else
            Help.ShowHelp(g.FormMain, Path, HelpNavigator.Topic)
        End If
    End Sub

    Public Shared Function Run(ByVal name As String, ByVal commandline As String) As Boolean
        Try
            Dim ProcInfo As New ProcessStartInfo
            ProcInfo.FileName = commandline
            If Str.Contains(commandline, ".exe") Then
                ProcInfo.FileName = Str.GetLeft(commandline, ".exe").TrimStart(""""c) + ".exe"
                ProcInfo.Arguments = Str.TrimStart(Str.TrimStart(Str.GetRight(commandline, ".exe"), """ "), " ")
            End If
            Dim Proc As Process = Process.Start(ProcInfo)
            Return True
        Catch
            Msg.Show(g.GetResource("The process couldn't be started") + vbCrLf + vbCrLf + _
                      "Name: " + name + vbCrLf + "Commandline: " + commandline, _
                      g.GetResource("Error"), MessageBoxIcon.Error)
        End Try
    End Function

    Public Shared Sub BlockPainting(ByVal control As Control)
        WinAPI.User32.SendMessage(control.Handle, &HB, 0, 0)
    End Sub

    Public Shared Sub ContinuePainting(ByVal control As Control)
        WinAPI.User32.SendMessage(control.Handle, &HB, 1, 0)
        control.Refresh()
    End Sub

    Public Shared Function Browse(ByVal text As String) As String
        Dim b As New FolderBrowserDialog
        b.Description = text
        b.ShowDialog()
        Return b.SelectedPath
    End Function

    Public Shared Function GetScriptBeforeSelection() As String
        Dim rtb As RichTextBox = g.FormMain.rtb
        Dim script As String
        For line As Integer = 0 To rtb.Lines.Length - 1
            If line < rtb.GetLineFromCharIndex(rtb.SelectionStart) Then
                Dim left As String = Str.GetLeft(rtb.Lines(line), "=")
                If left <> "" AndAlso Not Str.Contains(left, "(") AndAlso line = rtb.GetLineFromCharIndex(rtb.SelectionStart) - 1 Then
                    script += Str.GetRight(rtb.Lines(line), "=")
                Else
                    script += rtb.Lines(line) + vbCrLf
                End If
            End If
        Next
        Return script
    End Function

    Public Shared Sub StartPerformanceTest()
        PerformanceTestStart = Environment.TickCount
    End Sub

    Public Shared Sub ShowPerformanceTestResult()
        Msg.Show((Environment.TickCount - PerformanceTestStart).ToString, "Performance Test", MessageBoxIcon.Information)
    End Sub

    Public Shared Function GetMod(ByVal value As Integer) As Integer
        Dim modValue As Integer
        For Each x As Integer In New Integer() {1, 2, 4, 8, 16, 32, 64}
            If value Mod x = 0 Then modValue = x
        Next
        Return modValue
    End Function

    Public Shared Function GetRefTypeClone(ByVal o As Object) As Object
        Dim ms As MemoryStream = New MemoryStream
        Dim bf As BinaryFormatter = New BinaryFormatter
        bf.Serialize(ms, o)
        ms.Position = 0
        Return bf.Deserialize(ms)
        ms.Close()
    End Function
End Class